import enum


class PostgreDataType(enum.Enum):
    INT2 = "INT2"
    INT4 = "INT4"
    INT8 = "INT8"
    FLOAT4 = "FLOAT4"
    FLOAT8 = "FLOAT8"
    CHAR = "CHAR"
    VARCHAR = "VARCHAR"
    TEXT = "TEXT"
